@@ -436,21 +436,19 @@ test_that("rep_cols works as intended for listings and tables", {
   expect_equal(num_rep_cols(bmf), 0L) # repeated rowlabels are excluded from num_rep_cols
   expect_equal(num_rep_cols(blmf), 2L)
 
-  out <- export_as_txt(bmf, rep_cols = 2, cpp = 90) %>%
-    strsplit("\n") %>%
-    unlist()
+  strsplit_unlist <- function(x) {
+    unlist(strsplit(x, "\n"))
+  }
+
+  out <- strsplit_unlist(export_as_txt(bmf, rep_cols = 2, cpp = 90))
   expect_true(grepl(out[35], pattern = "mpg")) # mpg is repeated
   expect_true(grepl(out[35], pattern = "cyl")) # cyl is repeated
 
-  out <- export_as_txt(blmf, cpp = 70) %>%
-    strsplit("\n") %>%
-    unlist()
+  out <- strsplit_unlist(export_as_txt(blmf, cpp = 70))
   expect_true(grepl(out[51], pattern = "vs")) # vs is repeated
   expect_true(grepl(out[51], pattern = "gear")) # gear is repeated
 
-  out <- export_as_txt(blmf, rep_cols = 1, cpp = 70) %>%
-    strsplit("\n") %>%
-    unlist()
+  out <- strsplit_unlist(export_as_txt(blmf, rep_cols = 1, cpp = 70))
   expect_true(grepl(out[51], pattern = "vs")) # vs is repeated
   expect_false(grepl(out[51], pattern = "gear")) # gear is NOT repeated
 })
